# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import (
    messaging_profile_list_params,
    messaging_profile_create_params,
    messaging_profile_update_params,
    messaging_profile_list_short_codes_params,
    messaging_profile_retrieve_metrics_params,
    messaging_profile_list_phone_numbers_params,
    messaging_profile_list_alphanumeric_sender_ids_params,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from .autoresp_configs import (
    AutorespConfigsResource,
    AsyncAutorespConfigsResource,
    AutorespConfigsResourceWithRawResponse,
    AsyncAutorespConfigsResourceWithRawResponse,
    AutorespConfigsResourceWithStreamingResponse,
    AsyncAutorespConfigsResourceWithStreamingResponse,
)
from ...types.messaging_profile import MessagingProfile
from ...types.shared.short_code import ShortCode
from ...types.number_pool_settings_param import NumberPoolSettingsParam
from ...types.url_shortener_settings_param import URLShortenerSettingsParam
from ...types.messaging_profile_create_response import MessagingProfileCreateResponse
from ...types.messaging_profile_delete_response import MessagingProfileDeleteResponse
from ...types.messaging_profile_update_response import MessagingProfileUpdateResponse
from ...types.messaging_profile_retrieve_response import MessagingProfileRetrieveResponse
from ...types.messaging_profile_retrieve_metrics_response import MessagingProfileRetrieveMetricsResponse
from ...types.shared.phone_number_with_messaging_settings import PhoneNumberWithMessagingSettings
from ...types.messaging_profile_list_alphanumeric_sender_ids_response import (
    MessagingProfileListAlphanumericSenderIDsResponse,
)

__all__ = ["MessagingProfilesResource", "AsyncMessagingProfilesResource"]


class MessagingProfilesResource(SyncAPIResource):
    @cached_property
    def autoresp_configs(self) -> AutorespConfigsResource:
        return AutorespConfigsResource(self._client)

    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> MessagingProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MessagingProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagingProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MessagingProfilesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        whitelisted_destinations: SequenceNotStr[str],
        ai_assistant_id: Optional[str] | Omit = omit,
        alpha_sender: Optional[str] | Omit = omit,
        daily_spend_limit: str | Omit = omit,
        daily_spend_limit_enabled: bool | Omit = omit,
        enabled: bool | Omit = omit,
        health_webhook_url: Optional[str] | Omit = omit,
        mms_fall_back_to_sms: bool | Omit = omit,
        mms_transcoding: bool | Omit = omit,
        mobile_only: bool | Omit = omit,
        number_pool_settings: Optional[NumberPoolSettingsParam] | Omit = omit,
        resource_group_id: Optional[str] | Omit = omit,
        smart_encoding: bool | Omit = omit,
        url_shortener_settings: Optional[URLShortenerSettingsParam] | Omit = omit,
        webhook_api_version: Literal["1", "2", "2010-04-01"] | Omit = omit,
        webhook_failover_url: Optional[str] | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileCreateResponse:
        """
        Create a messaging profile

        Args:
          name: A user friendly name for the messaging profile.

          whitelisted_destinations: Destinations to which the messaging profile is allowed to send. The elements in
              the list must be valid ISO 3166-1 alpha-2 country codes. If set to `["*"]` all
              destinations will be allowed.

          ai_assistant_id: The AI assistant ID to associate with this messaging profile.

          alpha_sender: The alphanumeric sender ID to use when sending to destinations that require an
              alphanumeric sender ID.

          daily_spend_limit: The maximum amount of money (in USD) that can be spent by this profile before
              midnight UTC.

          daily_spend_limit_enabled: Whether to enforce the value configured by `daily_spend_limit`.

          enabled: Specifies whether the messaging profile is enabled or not.

          health_webhook_url: A URL to receive health check webhooks for numbers in this profile.

          mms_fall_back_to_sms: enables SMS fallback for MMS messages.

          mms_transcoding: enables automated resizing of MMS media.

          mobile_only: Send messages only to mobile phone numbers.

          number_pool_settings: Number Pool allows you to send messages from a pool of numbers of different
              types, assigning weights to each type. The pool consists of all the long code
              and toll free numbers assigned to the messaging profile.

              To disable this feature, set the object field to `null`.

          resource_group_id: The resource group ID to associate with this messaging profile.

          smart_encoding: Enables automatic character encoding optimization for SMS messages. When
              enabled, the system automatically selects the most efficient encoding (GSM-7 or
              UCS-2) based on message content to maximize character limits and minimize costs.

          url_shortener_settings: The URL shortener feature allows automatic replacement of URLs that were
              generated using a public URL shortener service. Some examples include bit.do,
              bit.ly, goo.gl, ht.ly, is.gd, ow.ly, rebrand.ly, t.co, tiny.cc, and tinyurl.com.
              Such URLs are replaced with with links generated by Telnyx. The use of custom
              links can improve branding and message deliverability.

              To disable this feature, set the object field to `null`.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1, v2, or a legacy
              2010-04-01 format.

          webhook_failover_url: The failover URL where webhooks related to this messaging profile will be sent
              if sending to the primary URL fails.

          webhook_url: The URL where webhooks related to this messaging profile will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messaging_profiles",
            body=maybe_transform(
                {
                    "name": name,
                    "whitelisted_destinations": whitelisted_destinations,
                    "ai_assistant_id": ai_assistant_id,
                    "alpha_sender": alpha_sender,
                    "daily_spend_limit": daily_spend_limit,
                    "daily_spend_limit_enabled": daily_spend_limit_enabled,
                    "enabled": enabled,
                    "health_webhook_url": health_webhook_url,
                    "mms_fall_back_to_sms": mms_fall_back_to_sms,
                    "mms_transcoding": mms_transcoding,
                    "mobile_only": mobile_only,
                    "number_pool_settings": number_pool_settings,
                    "resource_group_id": resource_group_id,
                    "smart_encoding": smart_encoding,
                    "url_shortener_settings": url_shortener_settings,
                    "webhook_api_version": webhook_api_version,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                messaging_profile_create_params.MessagingProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingProfileCreateResponse,
        )

    def retrieve(
        self,
        messaging_profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileRetrieveResponse:
        """
        Retrieve a messaging profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return self._get(
            f"/messaging_profiles/{messaging_profile_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingProfileRetrieveResponse,
        )

    def update(
        self,
        messaging_profile_id: str,
        *,
        alpha_sender: Optional[str] | Omit = omit,
        daily_spend_limit: str | Omit = omit,
        daily_spend_limit_enabled: bool | Omit = omit,
        enabled: bool | Omit = omit,
        mms_fall_back_to_sms: bool | Omit = omit,
        mms_transcoding: bool | Omit = omit,
        mobile_only: bool | Omit = omit,
        name: str | Omit = omit,
        number_pool_settings: Optional[NumberPoolSettingsParam] | Omit = omit,
        smart_encoding: bool | Omit = omit,
        url_shortener_settings: Optional[URLShortenerSettingsParam] | Omit = omit,
        v1_secret: str | Omit = omit,
        webhook_api_version: Literal["1", "2", "2010-04-01"] | Omit = omit,
        webhook_failover_url: Optional[str] | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        whitelisted_destinations: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileUpdateResponse:
        """
        Update a messaging profile

        Args:
          alpha_sender: The alphanumeric sender ID to use when sending to destinations that require an
              alphanumeric sender ID.

          daily_spend_limit: The maximum amount of money (in USD) that can be spent by this profile before
              midnight UTC.

          daily_spend_limit_enabled: Whether to enforce the value configured by `daily_spend_limit`.

          enabled: Specifies whether the messaging profile is enabled or not.

          mms_fall_back_to_sms: enables SMS fallback for MMS messages.

          mms_transcoding: enables automated resizing of MMS media.

          mobile_only: Send messages only to mobile phone numbers.

          name: A user friendly name for the messaging profile.

          number_pool_settings: Number Pool allows you to send messages from a pool of numbers of different
              types, assigning weights to each type. The pool consists of all the long code
              and toll free numbers assigned to the messaging profile.

              To disable this feature, set the object field to `null`.

          smart_encoding: Enables automatic character encoding optimization for SMS messages. When
              enabled, the system automatically selects the most efficient encoding (GSM-7 or
              UCS-2) based on message content to maximize character limits and minimize costs.

          url_shortener_settings: The URL shortener feature allows automatic replacement of URLs that were
              generated using a public URL shortener service. Some examples include bit.do,
              bit.ly, goo.gl, ht.ly, is.gd, ow.ly, rebrand.ly, t.co, tiny.cc, and tinyurl.com.
              Such URLs are replaced with with links generated by Telnyx. The use of custom
              links can improve branding and message deliverability.

              To disable this feature, set the object field to `null`.

          v1_secret: Secret used to authenticate with v1 endpoints.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1, v2, or a legacy
              2010-04-01 format.

          webhook_failover_url: The failover URL where webhooks related to this messaging profile will be sent
              if sending to the primary URL fails.

          webhook_url: The URL where webhooks related to this messaging profile will be sent.

          whitelisted_destinations: Destinations to which the messaging profile is allowed to send. The elements in
              the list must be valid ISO 3166-1 alpha-2 country codes. If set to `["*"]`, all
              destinations will be allowed.

              This field is required if the messaging profile doesn't have it defined yet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return self._patch(
            f"/messaging_profiles/{messaging_profile_id}",
            body=maybe_transform(
                {
                    "alpha_sender": alpha_sender,
                    "daily_spend_limit": daily_spend_limit,
                    "daily_spend_limit_enabled": daily_spend_limit_enabled,
                    "enabled": enabled,
                    "mms_fall_back_to_sms": mms_fall_back_to_sms,
                    "mms_transcoding": mms_transcoding,
                    "mobile_only": mobile_only,
                    "name": name,
                    "number_pool_settings": number_pool_settings,
                    "smart_encoding": smart_encoding,
                    "url_shortener_settings": url_shortener_settings,
                    "v1_secret": v1_secret,
                    "webhook_api_version": webhook_api_version,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "whitelisted_destinations": whitelisted_destinations,
                },
                messaging_profile_update_params.MessagingProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingProfileUpdateResponse,
        )

    def list(
        self,
        *,
        filter: messaging_profile_list_params.Filter | Omit = omit,
        filter_name_contains: str | Omit = omit,
        filter_name_eq: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MessagingProfile]:
        """
        List messaging profiles

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[name]

          filter_name_contains: Filter profiles by name containing the given string.

          filter_name_eq: Filter profiles by exact name match.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/messaging_profiles",
            page=SyncDefaultFlatPagination[MessagingProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "filter_name_contains": filter_name_contains,
                        "filter_name_eq": filter_name_eq,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    messaging_profile_list_params.MessagingProfileListParams,
                ),
            ),
            model=MessagingProfile,
        )

    def delete(
        self,
        messaging_profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileDeleteResponse:
        """
        Delete a messaging profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return self._delete(
            f"/messaging_profiles/{messaging_profile_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingProfileDeleteResponse,
        )

    def list_alphanumeric_sender_ids(
        self,
        id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MessagingProfileListAlphanumericSenderIDsResponse]:
        """
        List all alphanumeric sender IDs associated with a specific messaging profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/messaging_profiles/{id}/alphanumeric_sender_ids",
            page=SyncDefaultFlatPagination[MessagingProfileListAlphanumericSenderIDsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    messaging_profile_list_alphanumeric_sender_ids_params.MessagingProfileListAlphanumericSenderIDsParams,
                ),
            ),
            model=MessagingProfileListAlphanumericSenderIDsResponse,
        )

    def list_phone_numbers(
        self,
        messaging_profile_id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PhoneNumberWithMessagingSettings]:
        """
        List phone numbers associated with a messaging profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return self._get_api_list(
            f"/messaging_profiles/{messaging_profile_id}/phone_numbers",
            page=SyncDefaultFlatPagination[PhoneNumberWithMessagingSettings],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    messaging_profile_list_phone_numbers_params.MessagingProfileListPhoneNumbersParams,
                ),
            ),
            model=PhoneNumberWithMessagingSettings,
        )

    def list_short_codes(
        self,
        messaging_profile_id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ShortCode]:
        """
        List short codes associated with a messaging profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return self._get_api_list(
            f"/messaging_profiles/{messaging_profile_id}/short_codes",
            page=SyncDefaultFlatPagination[ShortCode],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    messaging_profile_list_short_codes_params.MessagingProfileListShortCodesParams,
                ),
            ),
            model=ShortCode,
        )

    def retrieve_metrics(
        self,
        id: str,
        *,
        time_frame: Literal["1h", "3h", "24h", "3d", "7d", "30d"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileRetrieveMetricsResponse:
        """
        Get detailed metrics for a specific messaging profile, broken down by time
        interval.

        Args:
          time_frame: The time frame for metrics.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/messaging_profiles/{id}/metrics",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"time_frame": time_frame},
                    messaging_profile_retrieve_metrics_params.MessagingProfileRetrieveMetricsParams,
                ),
            ),
            cast_to=MessagingProfileRetrieveMetricsResponse,
        )


class AsyncMessagingProfilesResource(AsyncAPIResource):
    @cached_property
    def autoresp_configs(self) -> AsyncAutorespConfigsResource:
        return AsyncAutorespConfigsResource(self._client)

    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMessagingProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagingProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagingProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessagingProfilesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        whitelisted_destinations: SequenceNotStr[str],
        ai_assistant_id: Optional[str] | Omit = omit,
        alpha_sender: Optional[str] | Omit = omit,
        daily_spend_limit: str | Omit = omit,
        daily_spend_limit_enabled: bool | Omit = omit,
        enabled: bool | Omit = omit,
        health_webhook_url: Optional[str] | Omit = omit,
        mms_fall_back_to_sms: bool | Omit = omit,
        mms_transcoding: bool | Omit = omit,
        mobile_only: bool | Omit = omit,
        number_pool_settings: Optional[NumberPoolSettingsParam] | Omit = omit,
        resource_group_id: Optional[str] | Omit = omit,
        smart_encoding: bool | Omit = omit,
        url_shortener_settings: Optional[URLShortenerSettingsParam] | Omit = omit,
        webhook_api_version: Literal["1", "2", "2010-04-01"] | Omit = omit,
        webhook_failover_url: Optional[str] | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileCreateResponse:
        """
        Create a messaging profile

        Args:
          name: A user friendly name for the messaging profile.

          whitelisted_destinations: Destinations to which the messaging profile is allowed to send. The elements in
              the list must be valid ISO 3166-1 alpha-2 country codes. If set to `["*"]` all
              destinations will be allowed.

          ai_assistant_id: The AI assistant ID to associate with this messaging profile.

          alpha_sender: The alphanumeric sender ID to use when sending to destinations that require an
              alphanumeric sender ID.

          daily_spend_limit: The maximum amount of money (in USD) that can be spent by this profile before
              midnight UTC.

          daily_spend_limit_enabled: Whether to enforce the value configured by `daily_spend_limit`.

          enabled: Specifies whether the messaging profile is enabled or not.

          health_webhook_url: A URL to receive health check webhooks for numbers in this profile.

          mms_fall_back_to_sms: enables SMS fallback for MMS messages.

          mms_transcoding: enables automated resizing of MMS media.

          mobile_only: Send messages only to mobile phone numbers.

          number_pool_settings: Number Pool allows you to send messages from a pool of numbers of different
              types, assigning weights to each type. The pool consists of all the long code
              and toll free numbers assigned to the messaging profile.

              To disable this feature, set the object field to `null`.

          resource_group_id: The resource group ID to associate with this messaging profile.

          smart_encoding: Enables automatic character encoding optimization for SMS messages. When
              enabled, the system automatically selects the most efficient encoding (GSM-7 or
              UCS-2) based on message content to maximize character limits and minimize costs.

          url_shortener_settings: The URL shortener feature allows automatic replacement of URLs that were
              generated using a public URL shortener service. Some examples include bit.do,
              bit.ly, goo.gl, ht.ly, is.gd, ow.ly, rebrand.ly, t.co, tiny.cc, and tinyurl.com.
              Such URLs are replaced with with links generated by Telnyx. The use of custom
              links can improve branding and message deliverability.

              To disable this feature, set the object field to `null`.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1, v2, or a legacy
              2010-04-01 format.

          webhook_failover_url: The failover URL where webhooks related to this messaging profile will be sent
              if sending to the primary URL fails.

          webhook_url: The URL where webhooks related to this messaging profile will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messaging_profiles",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "whitelisted_destinations": whitelisted_destinations,
                    "ai_assistant_id": ai_assistant_id,
                    "alpha_sender": alpha_sender,
                    "daily_spend_limit": daily_spend_limit,
                    "daily_spend_limit_enabled": daily_spend_limit_enabled,
                    "enabled": enabled,
                    "health_webhook_url": health_webhook_url,
                    "mms_fall_back_to_sms": mms_fall_back_to_sms,
                    "mms_transcoding": mms_transcoding,
                    "mobile_only": mobile_only,
                    "number_pool_settings": number_pool_settings,
                    "resource_group_id": resource_group_id,
                    "smart_encoding": smart_encoding,
                    "url_shortener_settings": url_shortener_settings,
                    "webhook_api_version": webhook_api_version,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                messaging_profile_create_params.MessagingProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingProfileCreateResponse,
        )

    async def retrieve(
        self,
        messaging_profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileRetrieveResponse:
        """
        Retrieve a messaging profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return await self._get(
            f"/messaging_profiles/{messaging_profile_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingProfileRetrieveResponse,
        )

    async def update(
        self,
        messaging_profile_id: str,
        *,
        alpha_sender: Optional[str] | Omit = omit,
        daily_spend_limit: str | Omit = omit,
        daily_spend_limit_enabled: bool | Omit = omit,
        enabled: bool | Omit = omit,
        mms_fall_back_to_sms: bool | Omit = omit,
        mms_transcoding: bool | Omit = omit,
        mobile_only: bool | Omit = omit,
        name: str | Omit = omit,
        number_pool_settings: Optional[NumberPoolSettingsParam] | Omit = omit,
        smart_encoding: bool | Omit = omit,
        url_shortener_settings: Optional[URLShortenerSettingsParam] | Omit = omit,
        v1_secret: str | Omit = omit,
        webhook_api_version: Literal["1", "2", "2010-04-01"] | Omit = omit,
        webhook_failover_url: Optional[str] | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        whitelisted_destinations: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileUpdateResponse:
        """
        Update a messaging profile

        Args:
          alpha_sender: The alphanumeric sender ID to use when sending to destinations that require an
              alphanumeric sender ID.

          daily_spend_limit: The maximum amount of money (in USD) that can be spent by this profile before
              midnight UTC.

          daily_spend_limit_enabled: Whether to enforce the value configured by `daily_spend_limit`.

          enabled: Specifies whether the messaging profile is enabled or not.

          mms_fall_back_to_sms: enables SMS fallback for MMS messages.

          mms_transcoding: enables automated resizing of MMS media.

          mobile_only: Send messages only to mobile phone numbers.

          name: A user friendly name for the messaging profile.

          number_pool_settings: Number Pool allows you to send messages from a pool of numbers of different
              types, assigning weights to each type. The pool consists of all the long code
              and toll free numbers assigned to the messaging profile.

              To disable this feature, set the object field to `null`.

          smart_encoding: Enables automatic character encoding optimization for SMS messages. When
              enabled, the system automatically selects the most efficient encoding (GSM-7 or
              UCS-2) based on message content to maximize character limits and minimize costs.

          url_shortener_settings: The URL shortener feature allows automatic replacement of URLs that were
              generated using a public URL shortener service. Some examples include bit.do,
              bit.ly, goo.gl, ht.ly, is.gd, ow.ly, rebrand.ly, t.co, tiny.cc, and tinyurl.com.
              Such URLs are replaced with with links generated by Telnyx. The use of custom
              links can improve branding and message deliverability.

              To disable this feature, set the object field to `null`.

          v1_secret: Secret used to authenticate with v1 endpoints.

          webhook_api_version: Determines which webhook format will be used, Telnyx API v1, v2, or a legacy
              2010-04-01 format.

          webhook_failover_url: The failover URL where webhooks related to this messaging profile will be sent
              if sending to the primary URL fails.

          webhook_url: The URL where webhooks related to this messaging profile will be sent.

          whitelisted_destinations: Destinations to which the messaging profile is allowed to send. The elements in
              the list must be valid ISO 3166-1 alpha-2 country codes. If set to `["*"]`, all
              destinations will be allowed.

              This field is required if the messaging profile doesn't have it defined yet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return await self._patch(
            f"/messaging_profiles/{messaging_profile_id}",
            body=await async_maybe_transform(
                {
                    "alpha_sender": alpha_sender,
                    "daily_spend_limit": daily_spend_limit,
                    "daily_spend_limit_enabled": daily_spend_limit_enabled,
                    "enabled": enabled,
                    "mms_fall_back_to_sms": mms_fall_back_to_sms,
                    "mms_transcoding": mms_transcoding,
                    "mobile_only": mobile_only,
                    "name": name,
                    "number_pool_settings": number_pool_settings,
                    "smart_encoding": smart_encoding,
                    "url_shortener_settings": url_shortener_settings,
                    "v1_secret": v1_secret,
                    "webhook_api_version": webhook_api_version,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "whitelisted_destinations": whitelisted_destinations,
                },
                messaging_profile_update_params.MessagingProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingProfileUpdateResponse,
        )

    def list(
        self,
        *,
        filter: messaging_profile_list_params.Filter | Omit = omit,
        filter_name_contains: str | Omit = omit,
        filter_name_eq: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MessagingProfile, AsyncDefaultFlatPagination[MessagingProfile]]:
        """
        List messaging profiles

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[name]

          filter_name_contains: Filter profiles by name containing the given string.

          filter_name_eq: Filter profiles by exact name match.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/messaging_profiles",
            page=AsyncDefaultFlatPagination[MessagingProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "filter_name_contains": filter_name_contains,
                        "filter_name_eq": filter_name_eq,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    messaging_profile_list_params.MessagingProfileListParams,
                ),
            ),
            model=MessagingProfile,
        )

    async def delete(
        self,
        messaging_profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileDeleteResponse:
        """
        Delete a messaging profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return await self._delete(
            f"/messaging_profiles/{messaging_profile_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingProfileDeleteResponse,
        )

    def list_alphanumeric_sender_ids(
        self,
        id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[
        MessagingProfileListAlphanumericSenderIDsResponse,
        AsyncDefaultFlatPagination[MessagingProfileListAlphanumericSenderIDsResponse],
    ]:
        """
        List all alphanumeric sender IDs associated with a specific messaging profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/messaging_profiles/{id}/alphanumeric_sender_ids",
            page=AsyncDefaultFlatPagination[MessagingProfileListAlphanumericSenderIDsResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    messaging_profile_list_alphanumeric_sender_ids_params.MessagingProfileListAlphanumericSenderIDsParams,
                ),
            ),
            model=MessagingProfileListAlphanumericSenderIDsResponse,
        )

    def list_phone_numbers(
        self,
        messaging_profile_id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PhoneNumberWithMessagingSettings, AsyncDefaultFlatPagination[PhoneNumberWithMessagingSettings]]:
        """
        List phone numbers associated with a messaging profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return self._get_api_list(
            f"/messaging_profiles/{messaging_profile_id}/phone_numbers",
            page=AsyncDefaultFlatPagination[PhoneNumberWithMessagingSettings],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    messaging_profile_list_phone_numbers_params.MessagingProfileListPhoneNumbersParams,
                ),
            ),
            model=PhoneNumberWithMessagingSettings,
        )

    def list_short_codes(
        self,
        messaging_profile_id: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ShortCode, AsyncDefaultFlatPagination[ShortCode]]:
        """
        List short codes associated with a messaging profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not messaging_profile_id:
            raise ValueError(
                f"Expected a non-empty value for `messaging_profile_id` but received {messaging_profile_id!r}"
            )
        return self._get_api_list(
            f"/messaging_profiles/{messaging_profile_id}/short_codes",
            page=AsyncDefaultFlatPagination[ShortCode],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    messaging_profile_list_short_codes_params.MessagingProfileListShortCodesParams,
                ),
            ),
            model=ShortCode,
        )

    async def retrieve_metrics(
        self,
        id: str,
        *,
        time_frame: Literal["1h", "3h", "24h", "3d", "7d", "30d"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingProfileRetrieveMetricsResponse:
        """
        Get detailed metrics for a specific messaging profile, broken down by time
        interval.

        Args:
          time_frame: The time frame for metrics.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/messaging_profiles/{id}/metrics",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"time_frame": time_frame},
                    messaging_profile_retrieve_metrics_params.MessagingProfileRetrieveMetricsParams,
                ),
            ),
            cast_to=MessagingProfileRetrieveMetricsResponse,
        )


class MessagingProfilesResourceWithRawResponse:
    def __init__(self, messaging_profiles: MessagingProfilesResource) -> None:
        self._messaging_profiles = messaging_profiles

        self.create = to_raw_response_wrapper(
            messaging_profiles.create,
        )
        self.retrieve = to_raw_response_wrapper(
            messaging_profiles.retrieve,
        )
        self.update = to_raw_response_wrapper(
            messaging_profiles.update,
        )
        self.list = to_raw_response_wrapper(
            messaging_profiles.list,
        )
        self.delete = to_raw_response_wrapper(
            messaging_profiles.delete,
        )
        self.list_alphanumeric_sender_ids = to_raw_response_wrapper(
            messaging_profiles.list_alphanumeric_sender_ids,
        )
        self.list_phone_numbers = to_raw_response_wrapper(
            messaging_profiles.list_phone_numbers,
        )
        self.list_short_codes = to_raw_response_wrapper(
            messaging_profiles.list_short_codes,
        )
        self.retrieve_metrics = to_raw_response_wrapper(
            messaging_profiles.retrieve_metrics,
        )

    @cached_property
    def autoresp_configs(self) -> AutorespConfigsResourceWithRawResponse:
        return AutorespConfigsResourceWithRawResponse(self._messaging_profiles.autoresp_configs)

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._messaging_profiles.actions)


class AsyncMessagingProfilesResourceWithRawResponse:
    def __init__(self, messaging_profiles: AsyncMessagingProfilesResource) -> None:
        self._messaging_profiles = messaging_profiles

        self.create = async_to_raw_response_wrapper(
            messaging_profiles.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            messaging_profiles.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            messaging_profiles.update,
        )
        self.list = async_to_raw_response_wrapper(
            messaging_profiles.list,
        )
        self.delete = async_to_raw_response_wrapper(
            messaging_profiles.delete,
        )
        self.list_alphanumeric_sender_ids = async_to_raw_response_wrapper(
            messaging_profiles.list_alphanumeric_sender_ids,
        )
        self.list_phone_numbers = async_to_raw_response_wrapper(
            messaging_profiles.list_phone_numbers,
        )
        self.list_short_codes = async_to_raw_response_wrapper(
            messaging_profiles.list_short_codes,
        )
        self.retrieve_metrics = async_to_raw_response_wrapper(
            messaging_profiles.retrieve_metrics,
        )

    @cached_property
    def autoresp_configs(self) -> AsyncAutorespConfigsResourceWithRawResponse:
        return AsyncAutorespConfigsResourceWithRawResponse(self._messaging_profiles.autoresp_configs)

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._messaging_profiles.actions)


class MessagingProfilesResourceWithStreamingResponse:
    def __init__(self, messaging_profiles: MessagingProfilesResource) -> None:
        self._messaging_profiles = messaging_profiles

        self.create = to_streamed_response_wrapper(
            messaging_profiles.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            messaging_profiles.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            messaging_profiles.update,
        )
        self.list = to_streamed_response_wrapper(
            messaging_profiles.list,
        )
        self.delete = to_streamed_response_wrapper(
            messaging_profiles.delete,
        )
        self.list_alphanumeric_sender_ids = to_streamed_response_wrapper(
            messaging_profiles.list_alphanumeric_sender_ids,
        )
        self.list_phone_numbers = to_streamed_response_wrapper(
            messaging_profiles.list_phone_numbers,
        )
        self.list_short_codes = to_streamed_response_wrapper(
            messaging_profiles.list_short_codes,
        )
        self.retrieve_metrics = to_streamed_response_wrapper(
            messaging_profiles.retrieve_metrics,
        )

    @cached_property
    def autoresp_configs(self) -> AutorespConfigsResourceWithStreamingResponse:
        return AutorespConfigsResourceWithStreamingResponse(self._messaging_profiles.autoresp_configs)

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._messaging_profiles.actions)


class AsyncMessagingProfilesResourceWithStreamingResponse:
    def __init__(self, messaging_profiles: AsyncMessagingProfilesResource) -> None:
        self._messaging_profiles = messaging_profiles

        self.create = async_to_streamed_response_wrapper(
            messaging_profiles.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            messaging_profiles.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            messaging_profiles.update,
        )
        self.list = async_to_streamed_response_wrapper(
            messaging_profiles.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            messaging_profiles.delete,
        )
        self.list_alphanumeric_sender_ids = async_to_streamed_response_wrapper(
            messaging_profiles.list_alphanumeric_sender_ids,
        )
        self.list_phone_numbers = async_to_streamed_response_wrapper(
            messaging_profiles.list_phone_numbers,
        )
        self.list_short_codes = async_to_streamed_response_wrapper(
            messaging_profiles.list_short_codes,
        )
        self.retrieve_metrics = async_to_streamed_response_wrapper(
            messaging_profiles.retrieve_metrics,
        )

    @cached_property
    def autoresp_configs(self) -> AsyncAutorespConfigsResourceWithStreamingResponse:
        return AsyncAutorespConfigsResourceWithStreamingResponse(self._messaging_profiles.autoresp_configs)

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._messaging_profiles.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["PortoutListRejectionCodesResponse", "Data"]


class Data(BaseModel):
    code: Optional[int] = None

    description: Optional[str] = None

    reason_required: Optional[bool] = None


class PortoutListRejectionCodesResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["CustomSipHeader"]


class CustomSipHeader(BaseModel):
    name: str
    """The name of the header to add."""

    value: str
    """The value of the header."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionSendDtmfParams"]


class ActionSendDtmfParams(TypedDict, total=False):
    digits: Required[str]
    """DTMF digits to send.

    Valid digits are 0-9, A-D, \\**, and #. Pauses can be added using w (0.5s) and W
    (1s).
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    duration_millis: int
    """
    Specifies for how many milliseconds each digit will be played in the audio
    stream. Ranges from 100 to 500ms
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CallUpdateParams"]


class CallUpdateParams(TypedDict, total=False):
    account_sid: Required[str]

    fallback_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="FallbackMethod")]
    """HTTP request type used for `FallbackUrl`."""

    fallback_url: Annotated[str, PropertyInfo(alias="FallbackUrl")]
    """
    A failover URL for which Telnyx will retrieve the TeXML call instructions if the
    Url is not responding.
    """

    method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="Method")]
    """HTTP request type used for `Url`."""

    status: Annotated[str, PropertyInfo(alias="Status")]
    """The value to set the call status to.

    Setting the status to completed ends the call.
    """

    status_callback: Annotated[str, PropertyInfo(alias="StatusCallback")]
    """URL destination for Telnyx to send status callback events to for the call."""

    status_callback_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="StatusCallbackMethod")]
    """HTTP request type used for `StatusCallback`."""

    texml: Annotated[str, PropertyInfo(alias="Texml")]
    """TeXML to replace the current one with."""

    url: Annotated[str, PropertyInfo(alias="Url")]
    """
    The URL where TeXML will make a request to retrieve a new set of TeXML
    instructions to continue the call flow.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["StreamStreamingSidJsonResponse"]


class StreamStreamingSidJsonResponse(BaseModel):
    account_sid: Optional[str] = None

    call_sid: Optional[str] = None

    date_updated: Optional[datetime] = None

    sid: Optional[str] = None
    """Identifier of a resource."""

    status: Optional[Literal["stopped"]] = None
    """The status of the streaming."""

    uri: Optional[str] = None
    """The relative URI for this streaming resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["PortingOrderDocumentsParam"]


class PortingOrderDocumentsParam(TypedDict, total=False):
    """Can be specified directly or via the `requirement_group_id` parameter."""

    invoice: Optional[str]
    """Returned ID of the submitted Invoice via the Documents endpoint"""

    loa: Optional[str]
    """Returned ID of the submitted LOA via the Documents endpoint"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["CallControlApplicationInboundParam"]


class CallControlApplicationInboundParam(TypedDict, total=False):
    channel_limit: int
    """
    When set, this will limit the total number of inbound calls to phone numbers
    associated with this connection.
    """

    shaken_stir_enabled: bool
    """
    When enabled Telnyx will include Shaken/Stir data in the Webhook for new inbound
    calls.
    """

    sip_subdomain: str
    """
    Specifies a subdomain that can be used to receive Inbound calls to a Connection,
    in the same way a phone number is used, from a SIP endpoint. Example: the
    subdomain "example.sip.telnyx.com" can be called from any SIP endpoint by using
    the SIP URI "sip:@example.sip.telnyx.com" where the user part can be any
    alphanumeric value. Please note TLS encrypted calls are not allowed for
    subdomain calls.
    """

    sip_subdomain_receive_settings: Literal["only_my_connections", "from_anyone"]
    """
    This option can be enabled to receive calls from: "Anyone" (any SIP endpoint in
    the public Internet) or "Only my connections" (any connection assigned to the
    same Telnyx user).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.texml.accounts.conferences import participant_update_params, participant_participants_params
from .....types.texml.accounts.conferences.participant_update_response import ParticipantUpdateResponse
from .....types.texml.accounts.conferences.participant_retrieve_response import ParticipantRetrieveResponse
from .....types.texml.accounts.conferences.participant_participants_response import ParticipantParticipantsResponse
from .....types.texml.accounts.conferences.participant_retrieve_participants_response import (
    ParticipantRetrieveParticipantsResponse,
)

__all__ = ["ParticipantsResource", "AsyncParticipantsResource"]


class ParticipantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ParticipantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ParticipantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ParticipantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ParticipantsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        call_sid_or_participant_label: str,
        *,
        account_sid: str,
        conference_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParticipantRetrieveResponse:
        """
        Gets conference participant resource

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        if not call_sid_or_participant_label:
            raise ValueError(
                f"Expected a non-empty value for `call_sid_or_participant_label` but received {call_sid_or_participant_label!r}"
            )
        return self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants/{call_sid_or_participant_label}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ParticipantRetrieveResponse,
        )

    def update(
        self,
        call_sid_or_participant_label: str,
        *,
        account_sid: str,
        conference_sid: str,
        announce_method: Literal["GET", "POST"] | Omit = omit,
        announce_url: str | Omit = omit,
        beep_on_exit: bool | Omit = omit,
        call_sid_to_coach: str | Omit = omit,
        coaching: bool | Omit = omit,
        end_conference_on_exit: bool | Omit = omit,
        hold: bool | Omit = omit,
        hold_method: Literal["GET", "POST"] | Omit = omit,
        hold_url: str | Omit = omit,
        muted: bool | Omit = omit,
        wait_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParticipantUpdateResponse:
        """
        Updates a conference participant

        Args:
          announce_method: The HTTP method used to call the `AnnounceUrl`. Defaults to `POST`.

          announce_url: The URL to call to announce something to the participant. The URL may return an
              MP3 fileo a WAV file, or a TwiML document that contains `<Play>`, `<Say>`,
              `<Pause>`, or `<Redirect>` verbs.

          beep_on_exit: Whether to play a notification beep to the conference when the participant
              exits.

          call_sid_to_coach: The SID of the participant who is being coached. The participant being coached
              is the only participant who can hear the participant who is coaching.

          coaching: Whether the participant is coaching another call. When `true`, `CallSidToCoach`
              has to be given.

          end_conference_on_exit: Whether to end the conference when the participant leaves.

          hold: Whether the participant should be on hold.

          hold_method: The HTTP method to use when calling the `HoldUrl`.

          hold_url: The URL to be called using the `HoldMethod` for music that plays when the
              participant is on hold. The URL may return an MP3 file, a WAV file, or a TwiML
              document that contains `<Play>`, `<Say>`, `<Pause>`, or `<Redirect>` verbs.

          muted: Whether the participant should be muted.

          wait_url: The URL to call for an audio file to play while the participant is waiting for
              the conference to start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        if not call_sid_or_participant_label:
            raise ValueError(
                f"Expected a non-empty value for `call_sid_or_participant_label` but received {call_sid_or_participant_label!r}"
            )
        return self._post(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants/{call_sid_or_participant_label}",
            body=maybe_transform(
                {
                    "announce_method": announce_method,
                    "announce_url": announce_url,
                    "beep_on_exit": beep_on_exit,
                    "call_sid_to_coach": call_sid_to_coach,
                    "coaching": coaching,
                    "end_conference_on_exit": end_conference_on_exit,
                    "hold": hold,
                    "hold_method": hold_method,
                    "hold_url": hold_url,
                    "muted": muted,
                    "wait_url": wait_url,
                },
                participant_update_params.ParticipantUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ParticipantUpdateResponse,
        )

    def delete(
        self,
        call_sid_or_participant_label: str,
        *,
        account_sid: str,
        conference_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a conference participant

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        if not call_sid_or_participant_label:
            raise ValueError(
                f"Expected a non-empty value for `call_sid_or_participant_label` but received {call_sid_or_participant_label!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants/{call_sid_or_participant_label}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def participants(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        amd_status_callback: str | Omit = omit,
        amd_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        application_sid: str | Omit = omit,
        beep: Literal["true", "false", "onEnter", "onExit"] | Omit = omit,
        caller_id: str | Omit = omit,
        call_sid_to_coach: str | Omit = omit,
        cancel_playback_on_detect_message_end: bool | Omit = omit,
        cancel_playback_on_machine_detection: bool | Omit = omit,
        coaching: bool | Omit = omit,
        conference_record: Literal["true", "false", "record-from-start", "do-not-record"] | Omit = omit,
        conference_recording_status_callback: str | Omit = omit,
        conference_recording_status_callback_event: str | Omit = omit,
        conference_recording_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        conference_recording_timeout: int | Omit = omit,
        conference_status_callback: str | Omit = omit,
        conference_status_callback_event: str | Omit = omit,
        conference_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        conference_trim: Literal["trim-silence", "do-not-trim"] | Omit = omit,
        custom_headers: Iterable[participant_participants_params.CustomHeader] | Omit = omit,
        early_media: bool | Omit = omit,
        end_conference_on_exit: bool | Omit = omit,
        from_: str | Omit = omit,
        label: str | Omit = omit,
        machine_detection: Literal["Enable", "DetectMessageEnd"] | Omit = omit,
        machine_detection_silence_timeout: int | Omit = omit,
        machine_detection_speech_end_threshold: int | Omit = omit,
        machine_detection_speech_threshold: int | Omit = omit,
        machine_detection_timeout: int | Omit = omit,
        max_participants: int | Omit = omit,
        muted: bool | Omit = omit,
        preferred_codecs: str | Omit = omit,
        record: bool | Omit = omit,
        recording_channels: Literal["mono", "dual"] | Omit = omit,
        recording_status_callback: str | Omit = omit,
        recording_status_callback_event: str | Omit = omit,
        recording_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        recording_track: Literal["inbound", "outbound", "both"] | Omit = omit,
        sip_auth_password: str | Omit = omit,
        sip_auth_username: str | Omit = omit,
        start_conference_on_enter: bool | Omit = omit,
        status_callback: str | Omit = omit,
        status_callback_event: str | Omit = omit,
        status_callback_method: Literal["GET", "POST"] | Omit = omit,
        time_limit: int | Omit = omit,
        timeout_seconds: int | Omit = omit,
        to: str | Omit = omit,
        trim: Literal["trim-silence", "do-not-trim"] | Omit = omit,
        wait_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParticipantParticipantsResponse:
        """
        Dials a new conference participant

        Args:
          amd_status_callback: The URL the result of answering machine detection will be sent to.

          amd_status_callback_method: HTTP request type used for `AmdStatusCallback`. Defaults to `POST`.

          application_sid: The SID of the TeXML application that will handle the new participant's call.
              Required unless joining an existing conference by its ConferenceSid.

          beep: Whether to play a notification beep to the conference when the participant
              enters and exits.

          caller_id: To be used as the caller id name (SIP From Display Name) presented to the
              destination (`To` number). The string should have a maximum of 128 characters,
              containing only letters, numbers, spaces, and `-_~!.+` special characters. If
              ommited, the display name will be the same as the number in the `From` field.

          call_sid_to_coach: The SID of the participant who is being coached. The participant being coached
              is the only participant who can hear the participant who is coaching.

          cancel_playback_on_detect_message_end: Whether to cancel ongoing playback on `greeting ended` detection. Defaults to
              `true`.

          cancel_playback_on_machine_detection: Whether to cancel ongoing playback on `machine` detection. Defaults to `true`.

          coaching: Whether the participant is coaching another call. When `true`, `CallSidToCoach`
              has to be given.

          conference_record: Whether to record the conference the participant is joining. Defualts to
              `do-not-record`. The boolean values `true` and `false` are synonymous with
              `record-from-start` and `do-not-record` respectively.

          conference_recording_status_callback: The URL the conference recording callbacks will be sent to.

          conference_recording_status_callback_event: The changes to the conference recording's state that should generate a call to
              `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
              Separate multiple values with a space. Defaults to `completed`. `failed` and
              `absent` are synonymous.

          conference_recording_status_callback_method: HTTP request type used for `ConferenceRecordingStatusCallback`. Defaults to
              `POST`.

          conference_recording_timeout: The number of seconds that Telnyx will wait for the recording to be stopped if
              silence is detected. The timer only starts when the speech is detected. Please
              note that the transcription is used to detect silence and the related charge
              will be applied. The minimum value is 0. The default value is 0 (infinite)

          conference_status_callback: The URL the conference callbacks will be sent to.

          conference_status_callback_event: The changes to the conference's state that should generate a call to
              `ConferenceStatusCallback`. Can be: `start`, `end`, `join` and `leave`. Separate
              multiple values with a space. By default no callbacks are sent.

          conference_status_callback_method: HTTP request type used for `ConferenceStatusCallback`. Defaults to `POST`.

          conference_trim: Whether to trim any leading and trailing silence from the conference recording.
              Defaults to `trim-silence`.

          custom_headers: Custom HTTP headers to be sent with the call. Each header should be an object
              with 'name' and 'value' properties.

          early_media: Whether participant shall be bridged to conference before the participant
              answers (from early media if available). Defaults to `false`.

          end_conference_on_exit: Whether to end the conference when the participant leaves. Defaults to `false`.

          from_: The phone number of the party that initiated the call. Phone numbers are
              formatted with a `+` and country code.

          label: A unique label for the participant that will be added to the conference. The
              label can be used to reference the participant for updates via the TeXML REST
              API.

          machine_detection: Whether to detect if a human or an answering machine picked up the call. Use
              `Enable` if you would like to ne notified as soon as the called party is
              identified. Use `DetectMessageEnd`, if you would like to leave a message on an
              answering machine.

          machine_detection_silence_timeout: If initial silence duration is greater than this value, consider it a machine.
              Ignored when `premium` detection is used.

          machine_detection_speech_end_threshold: Silence duration threshold after a greeting message or voice for it be
              considered human. Ignored when `premium` detection is used.

          machine_detection_speech_threshold: Maximum threshold of a human greeting. If greeting longer than this value,
              considered machine. Ignored when `premium` detection is used.

          machine_detection_timeout: How long answering machine detection should go on for before sending an
              `Unknown` result. Given in milliseconds.

          max_participants: The maximum number of participants in the conference. Can be a positive integer
              from 2 to 800. The default value is 250.

          muted: Whether the participant should be muted.

          preferred_codecs: The list of comma-separated codecs to be offered on a call.

          record: Whether to record the entire participant's call leg. Defaults to `false`.

          recording_channels: The number of channels in the final recording. Defaults to `mono`.

          recording_status_callback: The URL the recording callbacks will be sent to.

          recording_status_callback_event: The changes to the recording's state that should generate a call to
              `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
              Separate multiple values with a space. Defaults to `completed`.

          recording_status_callback_method: HTTP request type used for `RecordingStatusCallback`. Defaults to `POST`.

          recording_track: The audio track to record for the call. The default is `both`.

          sip_auth_password: The password to use for SIP authentication.

          sip_auth_username: The username to use for SIP authentication.

          start_conference_on_enter: Whether to start the conference when the participant enters. Defaults to `true`.

          status_callback: URL destination for Telnyx to send status callback events to for the call.

          status_callback_event: The changes to the call's state that should generate a call to `StatusCallback`.
              Can be: `initiated`, `ringing`, `answered`, and `completed`. Separate multiple
              values with a space. The default value is `completed`.

          status_callback_method: HTTP request type used for `StatusCallback`.

          time_limit: The maximum duration of the call in seconds.

          timeout_seconds: The number of seconds that we should allow the phone to ring before assuming
              there is no answer. Can be an integer between 5 and 120, inclusive. The default
              value is 30.

          to: The phone number of the called party. Phone numbers are formatted with a `+` and
              country code.

          trim: Whether to trim any leading and trailing silence from the recording. Defaults to
              `trim-silence`.

          wait_url: The URL to call for an audio file to play while the participant is waiting for
              the conference to start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return self._post(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants",
            body=maybe_transform(
                {
                    "amd_status_callback": amd_status_callback,
                    "amd_status_callback_method": amd_status_callback_method,
                    "application_sid": application_sid,
                    "beep": beep,
                    "caller_id": caller_id,
                    "call_sid_to_coach": call_sid_to_coach,
                    "cancel_playback_on_detect_message_end": cancel_playback_on_detect_message_end,
                    "cancel_playback_on_machine_detection": cancel_playback_on_machine_detection,
                    "coaching": coaching,
                    "conference_record": conference_record,
                    "conference_recording_status_callback": conference_recording_status_callback,
                    "conference_recording_status_callback_event": conference_recording_status_callback_event,
                    "conference_recording_status_callback_method": conference_recording_status_callback_method,
                    "conference_recording_timeout": conference_recording_timeout,
                    "conference_status_callback": conference_status_callback,
                    "conference_status_callback_event": conference_status_callback_event,
                    "conference_status_callback_method": conference_status_callback_method,
                    "conference_trim": conference_trim,
                    "custom_headers": custom_headers,
                    "early_media": early_media,
                    "end_conference_on_exit": end_conference_on_exit,
                    "from_": from_,
                    "label": label,
                    "machine_detection": machine_detection,
                    "machine_detection_silence_timeout": machine_detection_silence_timeout,
                    "machine_detection_speech_end_threshold": machine_detection_speech_end_threshold,
                    "machine_detection_speech_threshold": machine_detection_speech_threshold,
                    "machine_detection_timeout": machine_detection_timeout,
                    "max_participants": max_participants,
                    "muted": muted,
                    "preferred_codecs": preferred_codecs,
                    "record": record,
                    "recording_channels": recording_channels,
                    "recording_status_callback": recording_status_callback,
                    "recording_status_callback_event": recording_status_callback_event,
                    "recording_status_callback_method": recording_status_callback_method,
                    "recording_track": recording_track,
                    "sip_auth_password": sip_auth_password,
                    "sip_auth_username": sip_auth_username,
                    "start_conference_on_enter": start_conference_on_enter,
                    "status_callback": status_callback,
                    "status_callback_event": status_callback_event,
                    "status_callback_method": status_callback_method,
                    "time_limit": time_limit,
                    "timeout_seconds": timeout_seconds,
                    "to": to,
                    "trim": trim,
                    "wait_url": wait_url,
                },
                participant_participants_params.ParticipantParticipantsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ParticipantParticipantsResponse,
        )

    def retrieve_participants(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParticipantRetrieveParticipantsResponse:
        """
        Lists conference participants

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ParticipantRetrieveParticipantsResponse,
        )


class AsyncParticipantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncParticipantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncParticipantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncParticipantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncParticipantsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        call_sid_or_participant_label: str,
        *,
        account_sid: str,
        conference_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParticipantRetrieveResponse:
        """
        Gets conference participant resource

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        if not call_sid_or_participant_label:
            raise ValueError(
                f"Expected a non-empty value for `call_sid_or_participant_label` but received {call_sid_or_participant_label!r}"
            )
        return await self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants/{call_sid_or_participant_label}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ParticipantRetrieveResponse,
        )

    async def update(
        self,
        call_sid_or_participant_label: str,
        *,
        account_sid: str,
        conference_sid: str,
        announce_method: Literal["GET", "POST"] | Omit = omit,
        announce_url: str | Omit = omit,
        beep_on_exit: bool | Omit = omit,
        call_sid_to_coach: str | Omit = omit,
        coaching: bool | Omit = omit,
        end_conference_on_exit: bool | Omit = omit,
        hold: bool | Omit = omit,
        hold_method: Literal["GET", "POST"] | Omit = omit,
        hold_url: str | Omit = omit,
        muted: bool | Omit = omit,
        wait_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParticipantUpdateResponse:
        """
        Updates a conference participant

        Args:
          announce_method: The HTTP method used to call the `AnnounceUrl`. Defaults to `POST`.

          announce_url: The URL to call to announce something to the participant. The URL may return an
              MP3 fileo a WAV file, or a TwiML document that contains `<Play>`, `<Say>`,
              `<Pause>`, or `<Redirect>` verbs.

          beep_on_exit: Whether to play a notification beep to the conference when the participant
              exits.

          call_sid_to_coach: The SID of the participant who is being coached. The participant being coached
              is the only participant who can hear the participant who is coaching.

          coaching: Whether the participant is coaching another call. When `true`, `CallSidToCoach`
              has to be given.

          end_conference_on_exit: Whether to end the conference when the participant leaves.

          hold: Whether the participant should be on hold.

          hold_method: The HTTP method to use when calling the `HoldUrl`.

          hold_url: The URL to be called using the `HoldMethod` for music that plays when the
              participant is on hold. The URL may return an MP3 file, a WAV file, or a TwiML
              document that contains `<Play>`, `<Say>`, `<Pause>`, or `<Redirect>` verbs.

          muted: Whether the participant should be muted.

          wait_url: The URL to call for an audio file to play while the participant is waiting for
              the conference to start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        if not call_sid_or_participant_label:
            raise ValueError(
                f"Expected a non-empty value for `call_sid_or_participant_label` but received {call_sid_or_participant_label!r}"
            )
        return await self._post(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants/{call_sid_or_participant_label}",
            body=await async_maybe_transform(
                {
                    "announce_method": announce_method,
                    "announce_url": announce_url,
                    "beep_on_exit": beep_on_exit,
                    "call_sid_to_coach": call_sid_to_coach,
                    "coaching": coaching,
                    "end_conference_on_exit": end_conference_on_exit,
                    "hold": hold,
                    "hold_method": hold_method,
                    "hold_url": hold_url,
                    "muted": muted,
                    "wait_url": wait_url,
                },
                participant_update_params.ParticipantUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ParticipantUpdateResponse,
        )

    async def delete(
        self,
        call_sid_or_participant_label: str,
        *,
        account_sid: str,
        conference_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a conference participant

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        if not call_sid_or_participant_label:
            raise ValueError(
                f"Expected a non-empty value for `call_sid_or_participant_label` but received {call_sid_or_participant_label!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants/{call_sid_or_participant_label}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def participants(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        amd_status_callback: str | Omit = omit,
        amd_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        application_sid: str | Omit = omit,
        beep: Literal["true", "false", "onEnter", "onExit"] | Omit = omit,
        caller_id: str | Omit = omit,
        call_sid_to_coach: str | Omit = omit,
        cancel_playback_on_detect_message_end: bool | Omit = omit,
        cancel_playback_on_machine_detection: bool | Omit = omit,
        coaching: bool | Omit = omit,
        conference_record: Literal["true", "false", "record-from-start", "do-not-record"] | Omit = omit,
        conference_recording_status_callback: str | Omit = omit,
        conference_recording_status_callback_event: str | Omit = omit,
        conference_recording_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        conference_recording_timeout: int | Omit = omit,
        conference_status_callback: str | Omit = omit,
        conference_status_callback_event: str | Omit = omit,
        conference_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        conference_trim: Literal["trim-silence", "do-not-trim"] | Omit = omit,
        custom_headers: Iterable[participant_participants_params.CustomHeader] | Omit = omit,
        early_media: bool | Omit = omit,
        end_conference_on_exit: bool | Omit = omit,
        from_: str | Omit = omit,
        label: str | Omit = omit,
        machine_detection: Literal["Enable", "DetectMessageEnd"] | Omit = omit,
        machine_detection_silence_timeout: int | Omit = omit,
        machine_detection_speech_end_threshold: int | Omit = omit,
        machine_detection_speech_threshold: int | Omit = omit,
        machine_detection_timeout: int | Omit = omit,
        max_participants: int | Omit = omit,
        muted: bool | Omit = omit,
        preferred_codecs: str | Omit = omit,
        record: bool | Omit = omit,
        recording_channels: Literal["mono", "dual"] | Omit = omit,
        recording_status_callback: str | Omit = omit,
        recording_status_callback_event: str | Omit = omit,
        recording_status_callback_method: Literal["GET", "POST"] | Omit = omit,
        recording_track: Literal["inbound", "outbound", "both"] | Omit = omit,
        sip_auth_password: str | Omit = omit,
        sip_auth_username: str | Omit = omit,
        start_conference_on_enter: bool | Omit = omit,
        status_callback: str | Omit = omit,
        status_callback_event: str | Omit = omit,
        status_callback_method: Literal["GET", "POST"] | Omit = omit,
        time_limit: int | Omit = omit,
        timeout_seconds: int | Omit = omit,
        to: str | Omit = omit,
        trim: Literal["trim-silence", "do-not-trim"] | Omit = omit,
        wait_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParticipantParticipantsResponse:
        """
        Dials a new conference participant

        Args:
          amd_status_callback: The URL the result of answering machine detection will be sent to.

          amd_status_callback_method: HTTP request type used for `AmdStatusCallback`. Defaults to `POST`.

          application_sid: The SID of the TeXML application that will handle the new participant's call.
              Required unless joining an existing conference by its ConferenceSid.

          beep: Whether to play a notification beep to the conference when the participant
              enters and exits.

          caller_id: To be used as the caller id name (SIP From Display Name) presented to the
              destination (`To` number). The string should have a maximum of 128 characters,
              containing only letters, numbers, spaces, and `-_~!.+` special characters. If
              ommited, the display name will be the same as the number in the `From` field.

          call_sid_to_coach: The SID of the participant who is being coached. The participant being coached
              is the only participant who can hear the participant who is coaching.

          cancel_playback_on_detect_message_end: Whether to cancel ongoing playback on `greeting ended` detection. Defaults to
              `true`.

          cancel_playback_on_machine_detection: Whether to cancel ongoing playback on `machine` detection. Defaults to `true`.

          coaching: Whether the participant is coaching another call. When `true`, `CallSidToCoach`
              has to be given.

          conference_record: Whether to record the conference the participant is joining. Defualts to
              `do-not-record`. The boolean values `true` and `false` are synonymous with
              `record-from-start` and `do-not-record` respectively.

          conference_recording_status_callback: The URL the conference recording callbacks will be sent to.

          conference_recording_status_callback_event: The changes to the conference recording's state that should generate a call to
              `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
              Separate multiple values with a space. Defaults to `completed`. `failed` and
              `absent` are synonymous.

          conference_recording_status_callback_method: HTTP request type used for `ConferenceRecordingStatusCallback`. Defaults to
              `POST`.

          conference_recording_timeout: The number of seconds that Telnyx will wait for the recording to be stopped if
              silence is detected. The timer only starts when the speech is detected. Please
              note that the transcription is used to detect silence and the related charge
              will be applied. The minimum value is 0. The default value is 0 (infinite)

          conference_status_callback: The URL the conference callbacks will be sent to.

          conference_status_callback_event: The changes to the conference's state that should generate a call to
              `ConferenceStatusCallback`. Can be: `start`, `end`, `join` and `leave`. Separate
              multiple values with a space. By default no callbacks are sent.

          conference_status_callback_method: HTTP request type used for `ConferenceStatusCallback`. Defaults to `POST`.

          conference_trim: Whether to trim any leading and trailing silence from the conference recording.
              Defaults to `trim-silence`.

          custom_headers: Custom HTTP headers to be sent with the call. Each header should be an object
              with 'name' and 'value' properties.

          early_media: Whether participant shall be bridged to conference before the participant
              answers (from early media if available). Defaults to `false`.

          end_conference_on_exit: Whether to end the conference when the participant leaves. Defaults to `false`.

          from_: The phone number of the party that initiated the call. Phone numbers are
              formatted with a `+` and country code.

          label: A unique label for the participant that will be added to the conference. The
              label can be used to reference the participant for updates via the TeXML REST
              API.

          machine_detection: Whether to detect if a human or an answering machine picked up the call. Use
              `Enable` if you would like to ne notified as soon as the called party is
              identified. Use `DetectMessageEnd`, if you would like to leave a message on an
              answering machine.

          machine_detection_silence_timeout: If initial silence duration is greater than this value, consider it a machine.
              Ignored when `premium` detection is used.

          machine_detection_speech_end_threshold: Silence duration threshold after a greeting message or voice for it be
              considered human. Ignored when `premium` detection is used.

          machine_detection_speech_threshold: Maximum threshold of a human greeting. If greeting longer than this value,
              considered machine. Ignored when `premium` detection is used.

          machine_detection_timeout: How long answering machine detection should go on for before sending an
              `Unknown` result. Given in milliseconds.

          max_participants: The maximum number of participants in the conference. Can be a positive integer
              from 2 to 800. The default value is 250.

          muted: Whether the participant should be muted.

          preferred_codecs: The list of comma-separated codecs to be offered on a call.

          record: Whether to record the entire participant's call leg. Defaults to `false`.

          recording_channels: The number of channels in the final recording. Defaults to `mono`.

          recording_status_callback: The URL the recording callbacks will be sent to.

          recording_status_callback_event: The changes to the recording's state that should generate a call to
              `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
              Separate multiple values with a space. Defaults to `completed`.

          recording_status_callback_method: HTTP request type used for `RecordingStatusCallback`. Defaults to `POST`.

          recording_track: The audio track to record for the call. The default is `both`.

          sip_auth_password: The password to use for SIP authentication.

          sip_auth_username: The username to use for SIP authentication.

          start_conference_on_enter: Whether to start the conference when the participant enters. Defaults to `true`.

          status_callback: URL destination for Telnyx to send status callback events to for the call.

          status_callback_event: The changes to the call's state that should generate a call to `StatusCallback`.
              Can be: `initiated`, `ringing`, `answered`, and `completed`. Separate multiple
              values with a space. The default value is `completed`.

          status_callback_method: HTTP request type used for `StatusCallback`.

          time_limit: The maximum duration of the call in seconds.

          timeout_seconds: The number of seconds that we should allow the phone to ring before assuming
              there is no answer. Can be an integer between 5 and 120, inclusive. The default
              value is 30.

          to: The phone number of the called party. Phone numbers are formatted with a `+` and
              country code.

          trim: Whether to trim any leading and trailing silence from the recording. Defaults to
              `trim-silence`.

          wait_url: The URL to call for an audio file to play while the participant is waiting for
              the conference to start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return await self._post(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants",
            body=await async_maybe_transform(
                {
                    "amd_status_callback": amd_status_callback,
                    "amd_status_callback_method": amd_status_callback_method,
                    "application_sid": application_sid,
                    "beep": beep,
                    "caller_id": caller_id,
                    "call_sid_to_coach": call_sid_to_coach,
                    "cancel_playback_on_detect_message_end": cancel_playback_on_detect_message_end,
                    "cancel_playback_on_machine_detection": cancel_playback_on_machine_detection,
                    "coaching": coaching,
                    "conference_record": conference_record,
                    "conference_recording_status_callback": conference_recording_status_callback,
                    "conference_recording_status_callback_event": conference_recording_status_callback_event,
                    "conference_recording_status_callback_method": conference_recording_status_callback_method,
                    "conference_recording_timeout": conference_recording_timeout,
                    "conference_status_callback": conference_status_callback,
                    "conference_status_callback_event": conference_status_callback_event,
                    "conference_status_callback_method": conference_status_callback_method,
                    "conference_trim": conference_trim,
                    "custom_headers": custom_headers,
                    "early_media": early_media,
                    "end_conference_on_exit": end_conference_on_exit,
                    "from_": from_,
                    "label": label,
                    "machine_detection": machine_detection,
                    "machine_detection_silence_timeout": machine_detection_silence_timeout,
                    "machine_detection_speech_end_threshold": machine_detection_speech_end_threshold,
                    "machine_detection_speech_threshold": machine_detection_speech_threshold,
                    "machine_detection_timeout": machine_detection_timeout,
                    "max_participants": max_participants,
                    "muted": muted,
                    "preferred_codecs": preferred_codecs,
                    "record": record,
                    "recording_channels": recording_channels,
                    "recording_status_callback": recording_status_callback,
                    "recording_status_callback_event": recording_status_callback_event,
                    "recording_status_callback_method": recording_status_callback_method,
                    "recording_track": recording_track,
                    "sip_auth_password": sip_auth_password,
                    "sip_auth_username": sip_auth_username,
                    "start_conference_on_enter": start_conference_on_enter,
                    "status_callback": status_callback,
                    "status_callback_event": status_callback_event,
                    "status_callback_method": status_callback_method,
                    "time_limit": time_limit,
                    "timeout_seconds": timeout_seconds,
                    "to": to,
                    "trim": trim,
                    "wait_url": wait_url,
                },
                participant_participants_params.ParticipantParticipantsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ParticipantParticipantsResponse,
        )

    async def retrieve_participants(
        self,
        conference_sid: str,
        *,
        account_sid: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParticipantRetrieveParticipantsResponse:
        """
        Lists conference participants

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not conference_sid:
            raise ValueError(f"Expected a non-empty value for `conference_sid` but received {conference_sid!r}")
        return await self._get(
            f"/texml/Accounts/{account_sid}/Conferences/{conference_sid}/Participants",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ParticipantRetrieveParticipantsResponse,
        )


class ParticipantsResourceWithRawResponse:
    def __init__(self, participants: ParticipantsResource) -> None:
        self._participants = participants

        self.retrieve = to_raw_response_wrapper(
            participants.retrieve,
        )
        self.update = to_raw_response_wrapper(
            participants.update,
        )
        self.delete = to_raw_response_wrapper(
            participants.delete,
        )
        self.participants = to_raw_response_wrapper(
            participants.participants,
        )
        self.retrieve_participants = to_raw_response_wrapper(
            participants.retrieve_participants,
        )


class AsyncParticipantsResourceWithRawResponse:
    def __init__(self, participants: AsyncParticipantsResource) -> None:
        self._participants = participants

        self.retrieve = async_to_raw_response_wrapper(
            participants.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            participants.update,
        )
        self.delete = async_to_raw_response_wrapper(
            participants.delete,
        )
        self.participants = async_to_raw_response_wrapper(
            participants.participants,
        )
        self.retrieve_participants = async_to_raw_response_wrapper(
            participants.retrieve_participants,
        )


class ParticipantsResourceWithStreamingResponse:
    def __init__(self, participants: ParticipantsResource) -> None:
        self._participants = participants

        self.retrieve = to_streamed_response_wrapper(
            participants.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            participants.update,
        )
        self.delete = to_streamed_response_wrapper(
            participants.delete,
        )
        self.participants = to_streamed_response_wrapper(
            participants.participants,
        )
        self.retrieve_participants = to_streamed_response_wrapper(
            participants.retrieve_participants,
        )


class AsyncParticipantsResourceWithStreamingResponse:
    def __init__(self, participants: AsyncParticipantsResource) -> None:
        self._participants = participants

        self.retrieve = async_to_streamed_response_wrapper(
            participants.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            participants.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            participants.delete,
        )
        self.participants = async_to_streamed_response_wrapper(
            participants.participants,
        )
        self.retrieve_participants = async_to_streamed_response_wrapper(
            participants.retrieve_participants,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SubNumberOrderUpdateRequirementGroupParams"]


class SubNumberOrderUpdateRequirementGroupParams(TypedDict, total=False):
    requirement_group_id: Required[str]
    """The ID of the requirement group to associate"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PrivateWirelessGatewayCreateParams"]


class PrivateWirelessGatewayCreateParams(TypedDict, total=False):
    name: Required[str]
    """The private wireless gateway name."""

    network_id: Required[str]
    """The identification of the related network resource."""

    region_code: str
    """The code of the region where the private wireless gateway will be assigned.

    A list of available regions can be found at the regions endpoint
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TranscriptionSettingsConfigParam"]


class TranscriptionSettingsConfigParam(TypedDict, total=False):
    eager_eot_threshold: float
    """Available only for deepgram/flux.

    Confidence threshold for eager end of turn detection. Must be lower than or
    equal to eot_threshold. Setting this equal to eot_threshold effectively disables
    eager end of turn.
    """

    eot_threshold: float
    """Available only for deepgram/flux.

    Confidence required to trigger an end of turn. Higher values = more reliable
    turn detection but slightly increased latency.
    """

    eot_timeout_ms: int
    """Available only for deepgram/flux.

    Maximum milliseconds of silence before forcing an end of turn, regardless of
    confidence.
    """

    numerals: bool

    smart_format: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import speech_to_text_transcribe_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["SpeechToTextResource", "AsyncSpeechToTextResource"]


class SpeechToTextResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpeechToTextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SpeechToTextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeechToTextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SpeechToTextResourceWithStreamingResponse(self)

    def transcribe(
        self,
        *,
        input_format: Literal["mp3", "wav"],
        transcription_engine: Literal["Azure", "Deepgram", "Google", "Telnyx"],
        interim_results: bool | Omit = omit,
        language: str | Omit = omit,
        model: Literal[
            "fast",
            "deepgram/nova-2",
            "deepgram/nova-3",
            "latest_long",
            "latest_short",
            "command_and_search",
            "phone_call",
            "video",
            "default",
            "medical_conversation",
            "medical_dictation",
            "openai/whisper-tiny",
            "openai/whisper-large-v3-turbo",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Transcribe audio streams to text over WebSocket.

        Args:
          input_format: The format of input audio stream.

          transcription_engine: The transcription engine to use for processing the audio stream.

          interim_results: Whether to receive interim transcription results.

          language: The language spoken in the audio stream.

          model: The specific model to use within the selected transcription engine.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            "/speech-to-text/transcription",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "input_format": input_format,
                        "transcription_engine": transcription_engine,
                        "interim_results": interim_results,
                        "language": language,
                        "model": model,
                    },
                    speech_to_text_transcribe_params.SpeechToTextTranscribeParams,
                ),
            ),
            cast_to=NoneType,
        )


class AsyncSpeechToTextResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpeechToTextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeechToTextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeechToTextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSpeechToTextResourceWithStreamingResponse(self)

    async def transcribe(
        self,
        *,
        input_format: Literal["mp3", "wav"],
        transcription_engine: Literal["Azure", "Deepgram", "Google", "Telnyx"],
        interim_results: bool | Omit = omit,
        language: str | Omit = omit,
        model: Literal[
            "fast",
            "deepgram/nova-2",
            "deepgram/nova-3",
            "latest_long",
            "latest_short",
            "command_and_search",
            "phone_call",
            "video",
            "default",
            "medical_conversation",
            "medical_dictation",
            "openai/whisper-tiny",
            "openai/whisper-large-v3-turbo",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Transcribe audio streams to text over WebSocket.

        Args:
          input_format: The format of input audio stream.

          transcription_engine: The transcription engine to use for processing the audio stream.

          interim_results: Whether to receive interim transcription results.

          language: The language spoken in the audio stream.

          model: The specific model to use within the selected transcription engine.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            "/speech-to-text/transcription",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "input_format": input_format,
                        "transcription_engine": transcription_engine,
                        "interim_results": interim_results,
                        "language": language,
                        "model": model,
                    },
                    speech_to_text_transcribe_params.SpeechToTextTranscribeParams,
                ),
            ),
            cast_to=NoneType,
        )


class SpeechToTextResourceWithRawResponse:
    def __init__(self, speech_to_text: SpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.transcribe = to_raw_response_wrapper(
            speech_to_text.transcribe,
        )


class AsyncSpeechToTextResourceWithRawResponse:
    def __init__(self, speech_to_text: AsyncSpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.transcribe = async_to_raw_response_wrapper(
            speech_to_text.transcribe,
        )


class SpeechToTextResourceWithStreamingResponse:
    def __init__(self, speech_to_text: SpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.transcribe = to_streamed_response_wrapper(
            speech_to_text.transcribe,
        )


class AsyncSpeechToTextResourceWithStreamingResponse:
    def __init__(self, speech_to_text: AsyncSpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.transcribe = async_to_streamed_response_wrapper(
            speech_to_text.transcribe,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["ParticipantUpdateParams"]


class ParticipantUpdateParams(TypedDict, total=False):
    account_sid: Required[str]

    conference_sid: Required[str]

    announce_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="AnnounceMethod")]
    """The HTTP method used to call the `AnnounceUrl`. Defaults to `POST`."""

    announce_url: Annotated[str, PropertyInfo(alias="AnnounceUrl")]
    """The URL to call to announce something to the participant.

    The URL may return an MP3 fileo a WAV file, or a TwiML document that contains
    `<Play>`, `<Say>`, `<Pause>`, or `<Redirect>` verbs.
    """

    beep_on_exit: Annotated[bool, PropertyInfo(alias="BeepOnExit")]
    """
    Whether to play a notification beep to the conference when the participant
    exits.
    """

    call_sid_to_coach: Annotated[str, PropertyInfo(alias="CallSidToCoach")]
    """The SID of the participant who is being coached.

    The participant being coached is the only participant who can hear the
    participant who is coaching.
    """

    coaching: Annotated[bool, PropertyInfo(alias="Coaching")]
    """Whether the participant is coaching another call.

    When `true`, `CallSidToCoach` has to be given.
    """

    end_conference_on_exit: Annotated[bool, PropertyInfo(alias="EndConferenceOnExit")]
    """Whether to end the conference when the participant leaves."""

    hold: Annotated[bool, PropertyInfo(alias="Hold")]
    """Whether the participant should be on hold."""

    hold_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="HoldMethod")]
    """The HTTP method to use when calling the `HoldUrl`."""

    hold_url: Annotated[str, PropertyInfo(alias="HoldUrl")]
    """
    The URL to be called using the `HoldMethod` for music that plays when the
    participant is on hold. The URL may return an MP3 file, a WAV file, or a TwiML
    document that contains `<Play>`, `<Say>`, `<Pause>`, or `<Redirect>` verbs.
    """

    muted: Annotated[bool, PropertyInfo(alias="Muted")]
    """Whether the participant should be muted."""

    wait_url: Annotated[str, PropertyInfo(alias="WaitUrl")]
    """
    The URL to call for an audio file to play while the participant is waiting for
    the conference to start.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MigrationSourceCreateParams", "ProviderAuth"]


class MigrationSourceCreateParams(TypedDict, total=False):
    bucket_name: Required[str]
    """Bucket name to migrate the data from."""

    provider: Required[Literal["aws", "telnyx"]]
    """Cloud provider from which to migrate data.

    Use 'telnyx' if you want to migrate data from one Telnyx bucket to another.
    """

    provider_auth: Required[ProviderAuth]

    source_region: str
    """
    For intra-Telnyx buckets migration, specify the source bucket region in this
    field.
    """


class ProviderAuth(TypedDict, total=False):
    access_key: str
    """AWS Access Key. For Telnyx-to-Telnyx migrations, use your Telnyx API key here."""

    secret_access_key: str
    """AWS Secret Access Key.

    For Telnyx-to-Telnyx migrations, use your Telnyx API key here as well.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .by_phone_number import (
    ByPhoneNumberResource,
    AsyncByPhoneNumberResource,
    ByPhoneNumberResourceWithRawResponse,
    AsyncByPhoneNumberResourceWithRawResponse,
    ByPhoneNumberResourceWithStreamingResponse,
    AsyncByPhoneNumberResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "ByPhoneNumberResource",
    "AsyncByPhoneNumberResource",
    "ByPhoneNumberResourceWithRawResponse",
    "AsyncByPhoneNumberResourceWithRawResponse",
    "ByPhoneNumberResourceWithStreamingResponse",
    "AsyncByPhoneNumberResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["IP"]


class IP(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    connection_id: Optional[str] = None
    """ID of the IP Connection to which this IP should be attached."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    ip_address: Optional[str] = None
    """IP adddress represented by this resource."""

    port: Optional[int] = None
    """Port to use when connecting to this IP."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionUpdateClientStateParams"]


class ActionUpdateClientStateParams(TypedDict, total=False):
    client_state: Required[str]
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["EmbeddingCreateParams"]


class EmbeddingCreateParams(TypedDict, total=False):
    bucket_name: Required[str]

    document_chunk_overlap_size: int

    document_chunk_size: int

    embedding_model: Literal["thenlper/gte-large", "intfloat/multilingual-e5-large"]
    """Supported models to vectorize and embed documents."""

    loader: Literal["default", "intercom"]
    """Supported types of custom document loaders for embeddings."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionVerifyParams"]


class ActionVerifyParams(TypedDict, total=False):
    code: Required[str]
    """This is the code the user submits for verification."""

    verify_profile_id: Required[str]
    """The identifier of the associated Verify profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["McpServerCreateParams"]


class McpServerCreateParams(TypedDict, total=False):
    name: Required[str]

    type: Required[str]

    url: Required[str]

    allowed_tools: Optional[SequenceNotStr[str]]

    api_key_ref: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["MessagingListParams"]


class MessagingListParams(TypedDict, total=False):
    filter_messaging_profile_id: Annotated[str, PropertyInfo(alias="filter[messaging_profile_id]")]
    """Filter by messaging profile ID."""

    filter_phone_number: Annotated[str, PropertyInfo(alias="filter[phone_number]")]
    """Filter by exact phone number (supports comma-separated list)."""

    filter_phone_number_contains: Annotated[str, PropertyInfo(alias="filter[phone_number][contains]")]
    """Filter by phone number substring."""

    filter_type: Annotated[Literal["tollfree", "longcode", "shortcode"], PropertyInfo(alias="filter[type]")]
    """Filter by phone number type."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort_phone_number: Annotated[Literal["asc", "desc"], PropertyInfo(alias="sort[phone_number]")]
    """Sort by phone number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    TextToSpeechListVoicesResponse,
)
from telnyx._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTextToSpeech:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_generate_speech(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/text-to-speech/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        text_to_speech = client.text_to_speech.generate_speech(
            text="text",
            voice="voice",
        )
        assert text_to_speech.is_closed
        assert text_to_speech.json() == {"foo": "bar"}
        assert cast(Any, text_to_speech.is_closed) is True
        assert isinstance(text_to_speech, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_generate_speech(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/text-to-speech/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        text_to_speech = client.text_to_speech.with_raw_response.generate_speech(
            text="text",
            voice="voice",
        )

        assert text_to_speech.is_closed is True
        assert text_to_speech.http_request.headers.get("X-Stainless-Lang") == "python"
        assert text_to_speech.json() == {"foo": "bar"}
        assert isinstance(text_to_speech, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_generate_speech(self, client: Telnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/text-to-speech/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.text_to_speech.with_streaming_response.generate_speech(
            text="text",
            voice="voice",
        ) as text_to_speech:
            assert not text_to_speech.is_closed
            assert text_to_speech.http_request.headers.get("X-Stainless-Lang") == "python"

            assert text_to_speech.json() == {"foo": "bar"}
            assert cast(Any, text_to_speech.is_closed) is True
            assert isinstance(text_to_speech, StreamedBinaryAPIResponse)

        assert cast(Any, text_to_speech.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_voices(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.list_voices()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_voices_with_all_params(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.list_voices(
            elevenlabs_api_key_ref="elevenlabs_api_key_ref",
            provider="aws",
        )
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_voices(self, client: Telnyx) -> None:
        response = client.text_to_speech.with_raw_response.list_voices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = response.parse()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_voices(self, client: Telnyx) -> None:
        with client.text_to_speech.with_streaming_response.list_voices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = response.parse()
            assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTextToSpeech:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_generate_speech(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/text-to-speech/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        text_to_speech = await async_client.text_to_speech.generate_speech(
            text="text",
            voice="voice",
        )
        assert text_to_speech.is_closed
        assert await text_to_speech.json() == {"foo": "bar"}
        assert cast(Any, text_to_speech.is_closed) is True
        assert isinstance(text_to_speech, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_generate_speech(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/text-to-speech/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        text_to_speech = await async_client.text_to_speech.with_raw_response.generate_speech(
            text="text",
            voice="voice",
        )

        assert text_to_speech.is_closed is True
        assert text_to_speech.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await text_to_speech.json() == {"foo": "bar"}
        assert isinstance(text_to_speech, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_generate_speech(self, async_client: AsyncTelnyx, respx_mock: MockRouter) -> None:
        respx_mock.post("/text-to-speech/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.text_to_speech.with_streaming_response.generate_speech(
            text="text",
            voice="voice",
        ) as text_to_speech:
            assert not text_to_speech.is_closed
            assert text_to_speech.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await text_to_speech.json() == {"foo": "bar"}
            assert cast(Any, text_to_speech.is_closed) is True
            assert isinstance(text_to_speech, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, text_to_speech.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_voices(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.list_voices()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_voices_with_all_params(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.list_voices(
            elevenlabs_api_key_ref="elevenlabs_api_key_ref",
            provider="aws",
        )
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_voices(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.text_to_speech.with_raw_response.list_voices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = await response.parse()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_voices(self, async_client: AsyncTelnyx) -> None:
        async with async_client.text_to_speech.with_streaming_response.list_voices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = await response.parse()
            assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["InsightSettingsParam"]


class InsightSettingsParam(TypedDict, total=False):
    insight_group_id: str
    """Reference to an Insight Group.

    Insights in this group will be run automatically for all the assistant's
    conversations.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["TelnyxDownstreamCampaign"]


class TelnyxDownstreamCampaign(BaseModel):
    """
    Campaign is generated by the 10DLC registry once the corresponding campaign request is approved. Each campaign is assigned a unique identifier - **campaignId**. Once a campaign is activated, limited information is published to the NetNumber OSR service for consumption by members of the ecosystem. When a campaign is suspended(reversible) or expired(non-reversible), campaign data is deleted from the OSR service.
    """

    tcr_brand_id: str = FieldInfo(alias="tcrBrandId")
    """Unique identifier assigned to the brand by the registry."""

    tcr_campaign_id: str = FieldInfo(alias="tcrCampaignId")
    """Unique identifier assigned to the campaign by the registry."""

    age_gated: Optional[bool] = FieldInfo(alias="ageGated", default=None)
    """Age gated content in campaign."""

    assigned_phone_numbers_count: Optional[float] = FieldInfo(alias="assignedPhoneNumbersCount", default=None)
    """Number of phone numbers associated with the campaign"""

    brand_display_name: Optional[str] = FieldInfo(alias="brandDisplayName", default=None)
    """Display or marketing name of the brand."""

    campaign_status: Optional[
        Literal[
            "TCR_PENDING",
            "TCR_SUSPENDED",
            "TCR_EXPIRED",
            "TCR_ACCEPTED",
            "TCR_FAILED",
            "TELNYX_ACCEPTED",
            "TELNYX_FAILED",
            "MNO_PENDING",
            "MNO_ACCEPTED",
            "MNO_REJECTED",
            "MNO_PROVISIONED",
            "MNO_PROVISIONING_FAILED",
        ]
    ] = FieldInfo(alias="campaignStatus", default=None)
    """Campaign status"""

    created_at: Optional[str] = FieldInfo(alias="createdAt", default=None)
    """Date and time that the brand was created at."""

    description: Optional[str] = None
    """Summary description of this campaign."""

    direct_lending: Optional[bool] = FieldInfo(alias="directLending", default=None)
    """Direct lending or loan arrangement."""

    embedded_link: Optional[bool] = FieldInfo(alias="embeddedLink", default=None)
    """Does message generated by the campaign include URL link in SMS?"""

    embedded_link_sample: Optional[str] = FieldInfo(alias="embeddedLinkSample", default=None)
    """Sample of an embedded link that will be sent to subscribers."""

    embedded_phone: Optional[bool] = FieldInfo(alias="embeddedPhone", default=None)
    """Does message generated by the campaign include phone number in SMS?"""

    failure_reasons: Optional[str] = FieldInfo(alias="failureReasons", default=None)
    """Failure reasons if campaign submission failed"""

    help_keywords: Optional[str] = FieldInfo(alias="helpKeywords", default=None)
    """Subscriber help keywords. Multiple keywords are comma separated without space."""

    help_message: Optional[str] = FieldInfo(alias="helpMessage", default=None)
    """Help message of the campaign."""

    is_number_pooling_enabled: Optional[bool] = FieldInfo(alias="isNumberPoolingEnabled", default=None)
    """
    Indicates whether the campaign has a T-Mobile number pool ID associated with it.
    """

    message_flow: Optional[str] = FieldInfo(alias="messageFlow", default=None)
    """Message flow description."""

    number_pool: Optional[bool] = FieldInfo(alias="numberPool", default=None)
    """Does campaign utilize pool of phone numbers?"""

    optin_keywords: Optional[str] = FieldInfo(alias="optinKeywords", default=None)
    """Subscriber opt-in keywords.

    Multiple keywords are comma separated without space.
    """

    optin_message: Optional[str] = FieldInfo(alias="optinMessage", default=None)
    """Subscriber opt-in message."""

    optout_keywords: Optional[str] = FieldInfo(alias="optoutKeywords", default=None)
    """Subscriber opt-out keywords.

    Multiple keywords are comma separated without space.
    """

    optout_message: Optional[str] = FieldInfo(alias="optoutMessage", default=None)
    """Subscriber opt-out message."""

    privacy_policy_link: Optional[str] = FieldInfo(alias="privacyPolicyLink", default=None)
    """Link to the campaign's privacy policy."""

    sample1: Optional[str] = None
    """Message sample. Some campaign tiers require 1 or more message samples."""

    sample2: Optional[str] = None
    """Message sample. Some campaign tiers require 2 or more message samples."""

    sample3: Optional[str] = None
    """Message sample. Some campaign tiers require 3 or more message samples."""

    sample4: Optional[str] = None
    """Message sample. Some campaign tiers require 4 or more message samples."""

    sample5: Optional[str] = None
    """Message sample. Some campaign tiers require 5 or more message samples."""

    subscriber_optin: Optional[bool] = FieldInfo(alias="subscriberOptin", default=None)
    """Does campaign require subscriber to opt-in before SMS is sent to subscriber?"""

    subscriber_optout: Optional[bool] = FieldInfo(alias="subscriberOptout", default=None)
    """Does campaign support subscriber opt-out keyword(s)?"""

    sub_usecases: Optional[List[str]] = FieldInfo(alias="subUsecases", default=None)
    """Campaign sub-usecases.

    Must be of defined valid sub-usecase types. Use `/10dlc/enum/usecase` operation
    to retrieve list of valid sub-usecases
    """

    terms_and_conditions: Optional[bool] = FieldInfo(alias="termsAndConditions", default=None)
    """Is terms & conditions accepted?"""

    terms_and_conditions_link: Optional[str] = FieldInfo(alias="termsAndConditionsLink", default=None)
    """Link to the campaign's terms and conditions."""

    updated_at: Optional[str] = FieldInfo(alias="updatedAt", default=None)
    """Date and time that the brand was last updated at."""

    usecase: Optional[str] = None
    """Campaign usecase.

    Must be of defined valid types. Use `/10dlc/enum/usecase` operation to retrieve
    usecases available for given brand.
    """

    webhook_failover_url: Optional[str] = FieldInfo(alias="webhookFailoverURL", default=None)
    """Failover webhook to which campaign status updates are sent."""

    webhook_url: Optional[str] = FieldInfo(alias="webhookURL", default=None)
    """Webhook to which campaign status updates are sent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["UploadPendingCountResponse", "Data"]


class Data(BaseModel):
    pending_numbers_count: Optional[int] = None
    """
    The count of phone numbers that are pending assignment to the external
    connection.
    """

    pending_orders_count: Optional[int] = None
    """The count of number uploads that have not yet been uploaded to Microsoft."""


class UploadPendingCountResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionSendDtmfParams"]


class ActionSendDtmfParams(TypedDict, total=False):
    digits: Required[str]
    """DTMF digits to send.

    Valid characters: 0-9, A-D, \\**, #, w (0.5s pause), W (1s pause).
    """

    call_control_ids: SequenceNotStr[str]
    """Array of participant call control IDs to send DTMF to.

    When empty, DTMF will be sent to all participants.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    Must be a valid Base-64 encoded string.
    """

    duration_millis: int
    """Duration of each DTMF digit in milliseconds."""
